# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .result_type import ResultType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class BiomarkerResult(pydantic.BaseModel):
    """
    Represent the schema for an individual biomarker result.
    """

    name: str
    slug: typing.Optional[str]
    value: float
    result: str
    type: ResultType
    unit: typing.Optional[str]
    timestamp: typing.Optional[dt.datetime]
    notes: typing.Optional[str]
    min_range_value: typing.Optional[float]
    max_range_value: typing.Optional[float]
    is_above_max_range: typing.Optional[bool]
    is_below_min_range: typing.Optional[bool]
    interpretation: typing.Optional[str]
    loinc: typing.Optional[str]
    loinc_slug: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class SourceAuthType(str, enum.Enum):
    """
    An enumeration.
    """

    OAUTH = "oauth"
    TEAM_OAUTH = "team_oauth"
    SDK = "sdk"
    PASSWORD = "password"
    EMAIL = "email"
    APP = "app"
    EMPTY = ""

    def visit(
        self,
        oauth: typing.Callable[[], T_Result],
        team_oauth: typing.Callable[[], T_Result],
        sdk: typing.Callable[[], T_Result],
        password: typing.Callable[[], T_Result],
        email: typing.Callable[[], T_Result],
        app: typing.Callable[[], T_Result],
        empty: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is SourceAuthType.OAUTH:
            return oauth()
        if self is SourceAuthType.TEAM_OAUTH:
            return team_oauth()
        if self is SourceAuthType.SDK:
            return sdk()
        if self is SourceAuthType.PASSWORD:
            return password()
        if self is SourceAuthType.EMAIL:
            return email()
        if self is SourceAuthType.APP:
            return app()
        if self is SourceAuthType.EMPTY:
            return empty()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .client_facing_sleep_stream import ClientFacingSleepStream
from .client_facing_source import ClientFacingSource

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingSleep(pydantic.BaseModel):
    user_id: str = pydantic.Field(
        description="User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api."
    )
    id: str
    date: dt.datetime = pydantic.Field(
        description="Date of the specified record, formatted as ISO8601 datetime string in UTC 00:00. Deprecated in favour of calendar_date."
    )
    calendar_date: str = pydantic.Field(
        description="Date of the sleep summary in the YYYY-mm-dd format. This generally matches the sleep end date."
    )
    bedtime_start: dt.datetime = pydantic.Field(description="UTC Time when the sleep period started")
    bedtime_stop: dt.datetime = pydantic.Field(description="UTC Time when the sleep period ended")
    timezone_offset: typing.Optional[int] = pydantic.Field(
        description="Timezone offset from UTC as seconds. For example, EEST (Eastern European Summer Time, +3h) is 10800. PST (Pacific Standard Time, -8h) is -28800::seconds"
    )
    duration: int = pydantic.Field(
        description="Total duration of the sleep period (sleep.duration = sleep.bedtime_end - sleep.bedtime_start)::seconds"
    )
    total: int = pydantic.Field(
        description="Total amount of sleep registered during the sleep period (sleep.total = sleep.rem + sleep.light + sleep.deep)::seconds"
    )
    awake: int = pydantic.Field(description="Total amount of awake time registered during the sleep period::seconds")
    light: int = pydantic.Field(description="Total amount of light sleep registered during the sleep period::seconds")
    rem: int = pydantic.Field(
        description="Total amount of REM sleep registered during the sleep period, minutes::seconds"
    )
    deep: int = pydantic.Field(
        description="Total amount of deep (N3) sleep registered during the sleep period::seconds"
    )
    score: typing.Optional[int] = pydantic.Field(
        description="A value between 1 and 100 representing how well the user slept. Currently only available for Withings, Oura, Whoop and Garmin::scalar"
    )
    hr_lowest: typing.Optional[int] = pydantic.Field(
        description="The lowest heart rate (5 minutes sliding average) registered during the sleep period::beats per minute"
    )
    hr_average: typing.Optional[int] = pydantic.Field(
        description="The average heart rate registered during the sleep period::beats per minute"
    )
    efficiency: typing.Optional[float] = pydantic.Field(
        description="Sleep efficiency is the percentage of the sleep period spent asleep (100% * sleep.total / sleep.duration)::perc"
    )
    latency: typing.Optional[int] = pydantic.Field(
        description="Detected latency from bedtime_start to the beginning of the first five minutes of persistent sleep::seconds"
    )
    temperature_delta: typing.Optional[float] = pydantic.Field(
        description="Skin temperature deviation from the long-term temperature average::celcius"
    )
    skin_temperature: typing.Optional[float] = pydantic.Field(description="The skin temperature::celcius")
    hr_dip: typing.Optional[float] = pydantic.Field(
        description='Sleeping Heart Rate Dip is the percentage difference between your average waking heart rate and your average sleeping heart rate. In health studies, a greater "dip" is typically seen as a positive indicator of overall health. Currently only available for Garmin::perc'
    )
    average_hrv: typing.Optional[float] = pydantic.Field(
        description="The average heart rate variability registered during the sleep period::rmssd"
    )
    respiratory_rate: typing.Optional[float] = pydantic.Field(
        description="Average respiratory rate::breaths per minute"
    )
    source: ClientFacingSource = pydantic.Field(description="Source the data has come from.")
    sleep_stream: typing.Optional[ClientFacingSleepStream]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AppointmentProvider(str, enum.Enum):
    """
    An enumeration.
    """

    GETLABS = "getlabs"
    AXLEHEALTH = "axlehealth"

    def visit(self, getlabs: typing.Callable[[], T_Result], axlehealth: typing.Callable[[], T_Result]) -> T_Result:
        if self is AppointmentProvider.GETLABS:
            return getlabs()
        if self is AppointmentProvider.AXLEHEALTH:
            return axlehealth()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.consent import Consent
from ...types.http_validation_error import HttpValidationError
from ...types.patient_address_compatible import PatientAddressCompatible
from ...types.patient_details import PatientDetails
from ...types.physician_create_request_base import PhysicianCreateRequestBase
from ...types.post_order_response import PostOrderResponse
from ...types.shipping_address import ShippingAddress

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TestkitClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def register(
        self,
        *,
        user_id: str,
        sample_id: str,
        patient_details: PatientDetails,
        patient_address: PatientAddressCompatible,
        physician: typing.Optional[PhysicianCreateRequestBase] = OMIT,
        consents: typing.Optional[typing.List[Consent]] = OMIT,
    ) -> PostOrderResponse:
        """
        Parameters:
            - user_id: str.

            - sample_id: str.

            - patient_details: PatientDetails.

            - patient_address: PatientAddressCompatible.

            - physician: typing.Optional[PhysicianCreateRequestBase].

            - consents: typing.Optional[typing.List[Consent]].
        """
        _request: typing.Dict[str, typing.Any] = {
            "user_id": user_id,
            "sample_id": sample_id,
            "patient_details": patient_details,
            "patient_address": patient_address,
        }
        if physician is not OMIT:
            _request["physician"] = physician
        if consents is not OMIT:
            _request["consents"] = consents
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/order/testkit/register"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PostOrderResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_order(self, *, user_id: str, lab_test_id: str, shipping_details: ShippingAddress) -> PostOrderResponse:
        """
        Creates an order for an unregistered testkit

        Parameters:
            - user_id: str.

            - lab_test_id: str.

            - shipping_details: ShippingAddress.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/order/testkit"),
            json=jsonable_encoder(
                {"user_id": user_id, "lab_test_id": lab_test_id, "shipping_details": shipping_details}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PostOrderResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTestkitClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def register(
        self,
        *,
        user_id: str,
        sample_id: str,
        patient_details: PatientDetails,
        patient_address: PatientAddressCompatible,
        physician: typing.Optional[PhysicianCreateRequestBase] = OMIT,
        consents: typing.Optional[typing.List[Consent]] = OMIT,
    ) -> PostOrderResponse:
        """
        Parameters:
            - user_id: str.

            - sample_id: str.

            - patient_details: PatientDetails.

            - patient_address: PatientAddressCompatible.

            - physician: typing.Optional[PhysicianCreateRequestBase].

            - consents: typing.Optional[typing.List[Consent]].
        """
        _request: typing.Dict[str, typing.Any] = {
            "user_id": user_id,
            "sample_id": sample_id,
            "patient_details": patient_details,
            "patient_address": patient_address,
        }
        if physician is not OMIT:
            _request["physician"] = physician
        if consents is not OMIT:
            _request["consents"] = consents
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/order/testkit/register"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PostOrderResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_order(
        self, *, user_id: str, lab_test_id: str, shipping_details: ShippingAddress
    ) -> PostOrderResponse:
        """
        Creates an order for an unregistered testkit

        Parameters:
            - user_id: str.

            - lab_test_id: str.

            - shipping_details: ShippingAddress.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/order/testkit"),
            json=jsonable_encoder(
                {"user_id": user_id, "lab_test_id": lab_test_id, "shipping_details": shipping_details}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PostOrderResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...types.client_facing_provider_detailed import ClientFacingProviderDetailed

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ProvidersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all(self) -> typing.List[ClientFacingProviderDetailed]:
        """
        Get Provider list

        ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.providers.get_all()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/providers"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingProviderDetailed], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncProvidersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all(self) -> typing.List[ClientFacingProviderDetailed]:
        """
        Get Provider list

        ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.providers.get_all()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/providers"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingProviderDetailed], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ActivityV2InDb,
    Address,
    AppointmentAvailabilitySlots,
    AppointmentEventStatus,
    AppointmentProvider,
    AppointmentStatus,
    AppointmentType,
    AreaInfo,
    AttemptStatus,
    AuthType,
    BiomarkerResult,
    BodyV2InDb,
    ClientActivityResponse,
    ClientBodyResponse,
    ClientFacingActivity,
    ClientFacingApiKey,
    ClientFacingAppointment,
    ClientFacingAppointmentCancellationReason,
    ClientFacingAppointmentEvent,
    ClientFacingAtHomePhlebotomyOrder,
    ClientFacingAtHomePhlebotomyOrderDetails,
    ClientFacingBloodOxygenTimeseries,
    ClientFacingBloodPressureTimeseries,
    ClientFacingBody,
    ClientFacingCaffeineTimeseries,
    ClientFacingCaloriesActiveTimeseries,
    ClientFacingCaloriesBasalTimeseries,
    ClientFacingCholesterolTimeseries,
    ClientFacingDiagnosisInformation,
    ClientFacingDistanceTimeseries,
    ClientFacingElectrocardiogramVoltageTimeseries,
    ClientFacingFloorsClimbedTimeseries,
    ClientFacingFood,
    ClientFacingGlucoseTimeseries,
    ClientFacingHeartRate,
    ClientFacingHeartRateTimeseries,
    ClientFacingHrvTimeseries,
    ClientFacingHypnogramTimeseries,
    ClientFacingIgeTimeseries,
    ClientFacingIggTimeseries,
    ClientFacingLab,
    ClientFacingLabTest,
    ClientFacingLoinc,
    ClientFacingMarker,
    ClientFacingMarkerComplete,
    ClientFacingMealResponse,
    ClientFacingMindfulnessMinutesTimeseries,
    ClientFacingOrder,
    ClientFacingOrderDetails,
    ClientFacingOrderDetails_AtHomePhlebotomy,
    ClientFacingOrderDetails_Testkit,
    ClientFacingOrderDetails_WalkInTest,
    ClientFacingOrderEvent,
    ClientFacingPatientDetailsCompatible,
    ClientFacingPayorSearchResponse,
    ClientFacingPhysician,
    ClientFacingProfile,
    ClientFacingProvider,
    ClientFacingProviderDetailed,
    ClientFacingProviderWithStatus,
    ClientFacingResource,
    ClientFacingRespiratoryRateTimeseries,
    ClientFacingResult,
    ClientFacingShipment,
    ClientFacingSleep,
    ClientFacingSleepStream,
    ClientFacingSource,
    ClientFacingSport,
    ClientFacingStepsTimeseries,
    ClientFacingStream,
    ClientFacingTeam,
    ClientFacingTestKitOrderDetails,
    ClientFacingTestkitOrder,
    ClientFacingUser,
    ClientFacingUserKey,
    ClientFacingWalkInOrderDetails,
    ClientFacingWalkInTestOrder,
    ClientFacingWaterTimeseries,
    ClientFacingWorkout,
    ClientSleepResponse,
    ClientWorkoutResponse,
    ConnectedSourceClientFacing,
    ConnectionStatus,
    Consent,
    ConsentType,
    DaySlots,
    DelegatedFlowType,
    DemoConnectionStatus,
    DemoProviders,
    DeviceV2InDb,
    EmailProviders,
    Energy,
    EventDestinationPreferences,
    EventDestinationPreferencesEnabledItem,
    EventDestinationPreferencesPreferred,
    FallbackTimeZone,
    Fats,
    Gender,
    GetMarkersResponse,
    GetOrdersResponse,
    HealthInsuranceCreateRequest,
    HealthInsuranceCreateRequestBackImage,
    HealthInsuranceCreateRequestBackImage_ImageJpeg,
    HealthInsuranceCreateRequestBackImage_ImagePng,
    HealthInsuranceCreateRequestFrontImage,
    HealthInsuranceCreateRequestFrontImage_ImageJpeg,
    HealthInsuranceCreateRequestFrontImage_ImagePng,
    HealthInsuranceCreateRequestPatientSignatureImage,
    HealthInsuranceCreateRequestPatientSignatureImage_ImageJpeg,
    HealthInsuranceCreateRequestPatientSignatureImage_ImagePng,
    HttpValidationError,
    Jpeg,
    LabResultsMetadata,
    LabResultsRaw,
    LabResultsRawResults,
    LabTestCollectionMethod,
    LabTestSampleType,
    LabTestStatus,
    LastAttempt,
    LibreConfig,
    LinkTokenExchangeResponse,
    LngLat,
    Macros,
    ManualProviders,
    MarkerType,
    MealInDbBaseClientFacingSource,
    MetricsResult,
    Micros,
    OAuthProviders,
    OrderStatus,
    OrderTopLevelStatus,
    PaginatedUsersResponse,
    PasswordProviders,
    PatientAddressCompatible,
    PatientDetails,
    PersonDetails,
    PhlebotomyAreaInfo,
    PhysicianCreateRequest,
    PhysicianCreateRequestBase,
    PhysicianCreateRequestSignatureImage,
    PhysicianCreateRequestSignatureImage_ImageJpeg,
    PhysicianCreateRequestSignatureImage_ImagePng,
    Png,
    PostOrderResponse,
    ProfileInDb,
    ProviderLinkResponse,
    Providers,
    RawActivity,
    RawBody,
    RawDevices,
    RawProfile,
    RawSleep,
    RawWorkout,
    Region,
    ResponsibleRelationship,
    ResultType,
    ShippingAddress,
    SingleResourceStatistics,
    SingleUserResourceResponse,
    SleepV2InDb,
    Source,
    SourceAuthType,
    SourceLink,
    SourceType,
    TeamConfig,
    TimeSlot,
    TimeseriesMetricPoint,
    TimeseriesResource,
    UsAddress,
    UserRefreshErrorResponse,
    UserRefreshSuccessResponse,
    UserResourcesResponse,
    UserSignInToken,
    UserSignInTokenResponse,
    UserSignInTokenResponseSignInToken,
    UserSuccessResponse,
    ValidationError,
    ValidationErrorLocItem,
    VitalTokenCreatedResponse,
    WorkoutV2InDb,
)
from .errors import BadRequestError, UnprocessableEntityError
from .resources import (
    activity,
    body,
    devices,
    insurance,
    introspect,
    lab_tests,
    link,
    meal,
    profile,
    providers,
    sleep,
    team,
    testkit,
    user,
    vitals,
    workouts,
)
from .environment import VitalEnvironment

__all__ = [
    "ActivityV2InDb",
    "Address",
    "AppointmentAvailabilitySlots",
    "AppointmentEventStatus",
    "AppointmentProvider",
    "AppointmentStatus",
    "AppointmentType",
    "AreaInfo",
    "AttemptStatus",
    "AuthType",
    "BadRequestError",
    "BiomarkerResult",
    "BodyV2InDb",
    "ClientActivityResponse",
    "ClientBodyResponse",
    "ClientFacingActivity",
    "ClientFacingApiKey",
    "ClientFacingAppointment",
    "ClientFacingAppointmentCancellationReason",
    "ClientFacingAppointmentEvent",
    "ClientFacingAtHomePhlebotomyOrder",
    "ClientFacingAtHomePhlebotomyOrderDetails",
    "ClientFacingBloodOxygenTimeseries",
    "ClientFacingBloodPressureTimeseries",
    "ClientFacingBody",
    "ClientFacingCaffeineTimeseries",
    "ClientFacingCaloriesActiveTimeseries",
    "ClientFacingCaloriesBasalTimeseries",
    "ClientFacingCholesterolTimeseries",
    "ClientFacingDiagnosisInformation",
    "ClientFacingDistanceTimeseries",
    "ClientFacingElectrocardiogramVoltageTimeseries",
    "ClientFacingFloorsClimbedTimeseries",
    "ClientFacingFood",
    "ClientFacingGlucoseTimeseries",
    "ClientFacingHeartRate",
    "ClientFacingHeartRateTimeseries",
    "ClientFacingHrvTimeseries",
    "ClientFacingHypnogramTimeseries",
    "ClientFacingIgeTimeseries",
    "ClientFacingIggTimeseries",
    "ClientFacingLab",
    "ClientFacingLabTest",
    "ClientFacingLoinc",
    "ClientFacingMarker",
    "ClientFacingMarkerComplete",
    "ClientFacingMealResponse",
    "ClientFacingMindfulnessMinutesTimeseries",
    "ClientFacingOrder",
    "ClientFacingOrderDetails",
    "ClientFacingOrderDetails_AtHomePhlebotomy",
    "ClientFacingOrderDetails_Testkit",
    "ClientFacingOrderDetails_WalkInTest",
    "ClientFacingOrderEvent",
    "ClientFacingPatientDetailsCompatible",
    "ClientFacingPayorSearchResponse",
    "ClientFacingPhysician",
    "ClientFacingProfile",
    "ClientFacingProvider",
    "ClientFacingProviderDetailed",
    "ClientFacingProviderWithStatus",
    "ClientFacingResource",
    "ClientFacingRespiratoryRateTimeseries",
    "ClientFacingResult",
    "ClientFacingShipment",
    "ClientFacingSleep",
    "ClientFacingSleepStream",
    "ClientFacingSource",
    "ClientFacingSport",
    "ClientFacingStepsTimeseries",
    "ClientFacingStream",
    "ClientFacingTeam",
    "ClientFacingTestKitOrderDetails",
    "ClientFacingTestkitOrder",
    "ClientFacingUser",
    "ClientFacingUserKey",
    "ClientFacingWalkInOrderDetails",
    "ClientFacingWalkInTestOrder",
    "ClientFacingWaterTimeseries",
    "ClientFacingWorkout",
    "ClientSleepResponse",
    "ClientWorkoutResponse",
    "ConnectedSourceClientFacing",
    "ConnectionStatus",
    "Consent",
    "ConsentType",
    "DaySlots",
    "DelegatedFlowType",
    "DemoConnectionStatus",
    "DemoProviders",
    "DeviceV2InDb",
    "EmailProviders",
    "Energy",
    "EventDestinationPreferences",
    "EventDestinationPreferencesEnabledItem",
    "EventDestinationPreferencesPreferred",
    "FallbackTimeZone",
    "Fats",
    "Gender",
    "GetMarkersResponse",
    "GetOrdersResponse",
    "HealthInsuranceCreateRequest",
    "HealthInsuranceCreateRequestBackImage",
    "HealthInsuranceCreateRequestBackImage_ImageJpeg",
    "HealthInsuranceCreateRequestBackImage_ImagePng",
    "HealthInsuranceCreateRequestFrontImage",
    "HealthInsuranceCreateRequestFrontImage_ImageJpeg",
    "HealthInsuranceCreateRequestFrontImage_ImagePng",
    "HealthInsuranceCreateRequestPatientSignatureImage",
    "HealthInsuranceCreateRequestPatientSignatureImage_ImageJpeg",
    "HealthInsuranceCreateRequestPatientSignatureImage_ImagePng",
    "HttpValidationError",
    "Jpeg",
    "LabResultsMetadata",
    "LabResultsRaw",
    "LabResultsRawResults",
    "LabTestCollectionMethod",
    "LabTestSampleType",
    "LabTestStatus",
    "LastAttempt",
    "LibreConfig",
    "LinkTokenExchangeResponse",
    "LngLat",
    "Macros",
    "ManualProviders",
    "MarkerType",
    "MealInDbBaseClientFacingSource",
    "MetricsResult",
    "Micros",
    "OAuthProviders",
    "OrderStatus",
    "OrderTopLevelStatus",
    "PaginatedUsersResponse",
    "PasswordProviders",
    "PatientAddressCompatible",
    "PatientDetails",
    "PersonDetails",
    "PhlebotomyAreaInfo",
    "PhysicianCreateRequest",
    "PhysicianCreateRequestBase",
    "PhysicianCreateRequestSignatureImage",
    "PhysicianCreateRequestSignatureImage_ImageJpeg",
    "PhysicianCreateRequestSignatureImage_ImagePng",
    "Png",
    "PostOrderResponse",
    "ProfileInDb",
    "ProviderLinkResponse",
    "Providers",
    "RawActivity",
    "RawBody",
    "RawDevices",
    "RawProfile",
    "RawSleep",
    "RawWorkout",
    "Region",
    "ResponsibleRelationship",
    "ResultType",
    "ShippingAddress",
    "SingleResourceStatistics",
    "SingleUserResourceResponse",
    "SleepV2InDb",
    "Source",
    "SourceAuthType",
    "SourceLink",
    "SourceType",
    "TeamConfig",
    "TimeSlot",
    "TimeseriesMetricPoint",
    "TimeseriesResource",
    "UnprocessableEntityError",
    "UsAddress",
    "UserRefreshErrorResponse",
    "UserRefreshSuccessResponse",
    "UserResourcesResponse",
    "UserSignInToken",
    "UserSignInTokenResponse",
    "UserSignInTokenResponseSignInToken",
    "UserSuccessResponse",
    "ValidationError",
    "ValidationErrorLocItem",
    "VitalEnvironment",
    "VitalTokenCreatedResponse",
    "WorkoutV2InDb",
    "activity",
    "body",
    "devices",
    "insurance",
    "introspect",
    "lab_tests",
    "link",
    "meal",
    "profile",
    "providers",
    "sleep",
    "team",
    "testkit",
    "user",
    "vitals",
    "workouts",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .jpeg import Jpeg
from .png import Png


class HealthInsuranceCreateRequestPatientSignatureImage_ImageJpeg(Jpeg):
    content_type: typing_extensions.Literal["image/jpeg"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class HealthInsuranceCreateRequestPatientSignatureImage_ImagePng(Png):
    content_type: typing_extensions.Literal["image/png"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


HealthInsuranceCreateRequestPatientSignatureImage = typing.Union[
    HealthInsuranceCreateRequestPatientSignatureImage_ImageJpeg,
    HealthInsuranceCreateRequestPatientSignatureImage_ImagePng,
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Region(str, enum.Enum):
    """
    An enumeration.
    """

    US = "us"
    EU = "eu"
    SG = "sg"
    DE = "de"
    AU = "au"
    BR = "br"
    NL = "nl"
    FR = "fr"
    CA = "ca"

    def visit(
        self,
        us: typing.Callable[[], T_Result],
        eu: typing.Callable[[], T_Result],
        sg: typing.Callable[[], T_Result],
        de: typing.Callable[[], T_Result],
        au: typing.Callable[[], T_Result],
        br: typing.Callable[[], T_Result],
        nl: typing.Callable[[], T_Result],
        fr: typing.Callable[[], T_Result],
        ca: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Region.US:
            return us()
        if self is Region.EU:
            return eu()
        if self is Region.SG:
            return sg()
        if self is Region.DE:
            return de()
        if self is Region.AU:
            return au()
        if self is Region.BR:
            return br()
        if self is Region.NL:
            return nl()
        if self is Region.FR:
            return fr()
        if self is Region.CA:
            return ca()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingSource(pydantic.BaseModel):
    """
    Source summarizes where a sample or a summary is sourced from.
    At minimum, the source provider is always included.
    """

    provider: str = pydantic.Field(description="Provider slug. e.g., `oura`, `fitbit`, `garmin`.")
    type: typing.Optional[str] = pydantic.Field(
        description="The type of the data source (app or device) by which the summary or the timeseries data were recorded. This defaults to `unknown` when Vital cannot extract or infer that information"
    )
    app_id: typing.Optional[str] = pydantic.Field(
        description="The identifier of the app which recorded this summary. This is only applicable to multi-source providers like Apple Health and Android Health Connect."
    )
    name: str = pydantic.Field(description="Deprecated. Subject to removal after 1 Jan 2024.")
    slug: str = pydantic.Field(description="Deprecated. Use `provider` instead. Subject to removal after 1 Jan 2024.")
    logo: str = pydantic.Field(description="Deprecated. Subject to removal after 1 Jan 2024.")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

from .user_sign_in_token import UserSignInToken

UserSignInTokenResponseSignInToken = typing.Union[UserSignInToken, str]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.client_facing_physician import ClientFacingPhysician
from ...types.client_facing_team import ClientFacingTeam
from ...types.client_facing_user import ClientFacingUser
from ...types.http_validation_error import HttpValidationError

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class TeamClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_link_config(self, *, vital_link_token: typing.Any) -> typing.Dict[str, typing.Any]:
        """
        Post teams.

        Parameters:
            - vital_link_token: typing.Any.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/link/config"),
            headers=remove_none_from_dict(
                {**self._client_wrapper.get_headers(), "x-vital-link-token": vital_link_token}
            ),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, team_id: str) -> ClientFacingTeam:
        """
        Get team.

        Parameters:
            - team_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/team/{team_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingTeam, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_user_by_id(self, *, query_id: typing.Optional[str] = None) -> typing.List[ClientFacingUser]:
        """
        Search team users by user_id

        Parameters:
            - query_id: typing.Optional[str].
        ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.team.get_user_by_id()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/users/search"),
            params=remove_none_from_dict({"query_id": query_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingUser], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_svix_url(self) -> typing.Dict[str, typing.Any]:
        """
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.team.get_svix_url()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/svix/url"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_source_priorities(
        self, *, data_type: typing.Optional[str] = None
    ) -> typing.List[typing.Dict[str, typing.Any]]:
        """
        GET source priorities.

        Parameters:
            - data_type: typing.Optional[str].
        ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.team.get_source_priorities()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/source/priorities"),
            params=remove_none_from_dict({"data_type": data_type}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[typing.Dict[str, typing.Any]], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_source_priorities(self, *, team_id: str) -> typing.List[typing.Dict[str, typing.Any]]:
        """
        Patch source priorities.

        Parameters:
            - team_id: str.
        ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.team.update_source_priorities(
            team_id="team-id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/source/priorities"),
            params=remove_none_from_dict({"team_id": team_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[typing.Dict[str, typing.Any]], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_physicians(self, team_id: str) -> typing.List[ClientFacingPhysician]:
        """
        Parameters:
            - team_id: str.
        ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.team.get_physicians(
            team_id="team-id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/team/{team_id}/physicians"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingPhysician], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTeamClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_link_config(self, *, vital_link_token: typing.Any) -> typing.Dict[str, typing.Any]:
        """
        Post teams.

        Parameters:
            - vital_link_token: typing.Any.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/link/config"),
            headers=remove_none_from_dict(
                {**self._client_wrapper.get_headers(), "x-vital-link-token": vital_link_token}
            ),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, team_id: str) -> ClientFacingTeam:
        """
        Get team.

        Parameters:
            - team_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/team/{team_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingTeam, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_user_by_id(self, *, query_id: typing.Optional[str] = None) -> typing.List[ClientFacingUser]:
        """
        Search team users by user_id

        Parameters:
            - query_id: typing.Optional[str].
        ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.team.get_user_by_id()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/users/search"),
            params=remove_none_from_dict({"query_id": query_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingUser], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_svix_url(self) -> typing.Dict[str, typing.Any]:
        """
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.team.get_svix_url()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/svix/url"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_source_priorities(
        self, *, data_type: typing.Optional[str] = None
    ) -> typing.List[typing.Dict[str, typing.Any]]:
        """
        GET source priorities.

        Parameters:
            - data_type: typing.Optional[str].
        ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.team.get_source_priorities()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/source/priorities"),
            params=remove_none_from_dict({"data_type": data_type}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[typing.Dict[str, typing.Any]], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_source_priorities(self, *, team_id: str) -> typing.List[typing.Dict[str, typing.Any]]:
        """
        Patch source priorities.

        Parameters:
            - team_id: str.
        ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.team.update_source_priorities(
            team_id="team-id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/team/source/priorities"),
            params=remove_none_from_dict({"team_id": team_id}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[typing.Dict[str, typing.Any]], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_physicians(self, team_id: str) -> typing.List[ClientFacingPhysician]:
        """
        Parameters:
            - team_id: str.
        ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.team.get_physicians(
            team_id="team-id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/team/{team_id}/physicians"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingPhysician], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.bad_request_error import BadRequestError
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.client_facing_provider_with_status import ClientFacingProviderWithStatus
from ...types.client_facing_user import ClientFacingUser
from ...types.client_facing_user_key import ClientFacingUserKey
from ...types.http_validation_error import HttpValidationError
from ...types.metrics_result import MetricsResult
from ...types.paginated_users_response import PaginatedUsersResponse
from ...types.providers import Providers
from ...types.user_refresh_error_response import UserRefreshErrorResponse
from ...types.user_refresh_success_response import UserRefreshSuccessResponse
from ...types.user_sign_in_token_response import UserSignInTokenResponse
from ...types.user_success_response import UserSuccessResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all(
        self, *, offset: typing.Optional[int] = None, limit: typing.Optional[int] = None
    ) -> PaginatedUsersResponse:
        """
        GET All users for team.

        Parameters:
            - offset: typing.Optional[int].

            - limit: typing.Optional[int].
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/user"),
            params=remove_none_from_dict({"offset": offset, "limit": limit}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PaginatedUsersResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, client_user_id: str, fallback_time_zone: typing.Optional[str] = OMIT) -> ClientFacingUserKey:
        """
        POST Create a Vital user given a client_user_id and returns the user_id.

        Parameters:
            - client_user_id: str. A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

            - fallback_time_zone: typing.Optional[str].
                                                            Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                                                            Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).

        """
        _request: typing.Dict[str, typing.Any] = {"client_user_id": client_user_id}
        if fallback_time_zone is not OMIT:
            _request["fallback_time_zone"] = fallback_time_zone
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/user"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingUserKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_team_metrics(self) -> MetricsResult:
        """
        GET metrics for team.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/user/metrics"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(MetricsResult, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_user_sign_in_token(self, user_id: str) -> UserSignInTokenResponse:
        """
        Parameters:
            - user_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}/sign_in_token"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserSignInTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_connected_providers(self, user_id: str) -> typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]:
        """
        GET Users connected providers

        Parameters:
            - user_id: str.
        ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_connected_providers(
            user_id="user-id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/providers/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.List[ClientFacingProviderWithStatus]], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, user_id: str) -> ClientFacingUser:
        """
        GET User details given the user_id.

        Parameters:
            - user_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingUser, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, user_id: str) -> UserSuccessResponse:
        """
        Parameters:
            - user_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserSuccessResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch(self, user_id: str, *, fallback_time_zone: typing.Optional[str] = OMIT) -> None:
        """
        Parameters:
            - user_id: str.

            - fallback_time_zone: typing.Optional[str].
                                                            Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                                                            Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).
                                                            ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.patch(
            user_id="user-id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if fallback_time_zone is not OMIT:
            _request["fallback_time_zone"] = fallback_time_zone
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_by_client_user_id(self, client_user_id: str) -> ClientFacingUser:
        """
        GET user_id from client_user_id.

        Parameters:
            - client_user_id: str. A unique ID representing the end user. Typically this will be a user ID number from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/resolve/{client_user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingUser, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deregister_provider(self, user_id: str, provider: Providers) -> UserSuccessResponse:
        """
        Parameters:
            - user_id: str.

            - provider: Providers. Provider slug. e.g., `oura`, `fitbit`, `garmin`.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}/{provider}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserSuccessResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def refresh(self, user_id: str) -> UserRefreshSuccessResponse:
        """
        Trigger a manual refresh for a specific user

        Parameters:
            - user_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/refresh/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserRefreshSuccessResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(UserRefreshErrorResponse, _response.json()))  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncUserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all(
        self, *, offset: typing.Optional[int] = None, limit: typing.Optional[int] = None
    ) -> PaginatedUsersResponse:
        """
        GET All users for team.

        Parameters:
            - offset: typing.Optional[int].

            - limit: typing.Optional[int].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/user"),
            params=remove_none_from_dict({"offset": offset, "limit": limit}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PaginatedUsersResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self, *, client_user_id: str, fallback_time_zone: typing.Optional[str] = OMIT
    ) -> ClientFacingUserKey:
        """
        POST Create a Vital user given a client_user_id and returns the user_id.

        Parameters:
            - client_user_id: str. A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

            - fallback_time_zone: typing.Optional[str].
                                                            Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                                                            Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).

        """
        _request: typing.Dict[str, typing.Any] = {"client_user_id": client_user_id}
        if fallback_time_zone is not OMIT:
            _request["fallback_time_zone"] = fallback_time_zone
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/user"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingUserKey, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_team_metrics(self) -> MetricsResult:
        """
        GET metrics for team.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v2/user/metrics"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(MetricsResult, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_user_sign_in_token(self, user_id: str) -> UserSignInTokenResponse:
        """
        Parameters:
            - user_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}/sign_in_token"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserSignInTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_connected_providers(
        self, user_id: str
    ) -> typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]:
        """
        GET Users connected providers

        Parameters:
            - user_id: str.
        ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.user.get_connected_providers(
            user_id="user-id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/providers/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Dict[str, typing.List[ClientFacingProviderWithStatus]], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, user_id: str) -> ClientFacingUser:
        """
        GET User details given the user_id.

        Parameters:
            - user_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingUser, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, user_id: str) -> UserSuccessResponse:
        """
        Parameters:
            - user_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserSuccessResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch(self, user_id: str, *, fallback_time_zone: typing.Optional[str] = OMIT) -> None:
        """
        Parameters:
            - user_id: str.

            - fallback_time_zone: typing.Optional[str].
                                                            Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                                                            Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).
                                                            ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.user.patch(
            user_id="user-id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if fallback_time_zone is not OMIT:
            _request["fallback_time_zone"] = fallback_time_zone
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_by_client_user_id(self, client_user_id: str) -> ClientFacingUser:
        """
        GET user_id from client_user_id.

        Parameters:
            - client_user_id: str. A unique ID representing the end user. Typically this will be a user ID number from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/resolve/{client_user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingUser, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deregister_provider(self, user_id: str, provider: Providers) -> UserSuccessResponse:
        """
        Parameters:
            - user_id: str.

            - provider: Providers. Provider slug. e.g., `oura`, `fitbit`, `garmin`.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/{user_id}/{provider}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserSuccessResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def refresh(self, user_id: str) -> UserRefreshSuccessResponse:
        """
        Trigger a manual refresh for a specific user

        Parameters:
            - user_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/user/refresh/{user_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UserRefreshSuccessResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(UserRefreshErrorResponse, _response.json()))  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .client_facing_lab_test import ClientFacingLabTest
from .client_facing_order_details import ClientFacingOrderDetails
from .client_facing_order_event import ClientFacingOrderEvent
from .client_facing_patient_details_compatible import ClientFacingPatientDetailsCompatible
from .client_facing_physician import ClientFacingPhysician
from .order_top_level_status import OrderTopLevelStatus
from .patient_address_compatible import PatientAddressCompatible
from .shipping_address import ShippingAddress

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingOrder(pydantic.BaseModel):
    user_id: str = pydantic.Field(
        description="User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api."
    )
    id: str = pydantic.Field(description="The Vital Order ID")
    team_id: str = pydantic.Field(description="Your team id.")
    patient_details: typing.Optional[ClientFacingPatientDetailsCompatible] = pydantic.Field(
        description="Patient Details"
    )
    patient_address: typing.Optional[PatientAddressCompatible] = pydantic.Field(description="Patient Address")
    lab_test: ClientFacingLabTest = pydantic.Field(description="The Vital Test associated with the order")
    details: ClientFacingOrderDetails
    sample_id: typing.Optional[str] = pydantic.Field(description="Sample ID")
    notes: typing.Optional[str] = pydantic.Field(description="Notes associated with the order")
    created_at: dt.datetime = pydantic.Field(description="When your order was created")
    updated_at: dt.datetime = pydantic.Field(description="When your order was last updated.")
    events: typing.List[ClientFacingOrderEvent]
    status: typing.Optional[OrderTopLevelStatus]
    physician: typing.Optional[ClientFacingPhysician]
    health_insurance_id: typing.Optional[str] = pydantic.Field(description="Vital ID of the health insurance.")
    requisition_form_url: typing.Optional[str] = pydantic.Field(description="DEPRECATED. Requistion form url.")
    priority: typing.Optional[bool] = pydantic.Field(
        description="Defines whether order is priority or not. Only available for Labcorp. For Labcorp, this corresponds to a STAT order."
    )
    shipping_details: typing.Optional[ShippingAddress] = pydantic.Field(
        description="Shipping Details. For unregistered testkit orders."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.client_facing_diagnosis_information import ClientFacingDiagnosisInformation
from ...types.client_facing_payor_search_response import ClientFacingPayorSearchResponse
from ...types.http_validation_error import HttpValidationError

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InsuranceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_payor_info(
        self, *, insurance_name: str, insurance_state: typing.Optional[str] = OMIT
    ) -> typing.List[ClientFacingPayorSearchResponse]:
        """
        Parameters:
            - insurance_name: str.

            - insurance_state: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {"insurance_name": insurance_name}
        if insurance_state is not OMIT:
            _request["insurance_state"] = insurance_state
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/insurance/search/payor"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingPayorSearchResponse], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_diagnosis(self, *, diagnosis_query: str) -> typing.List[ClientFacingDiagnosisInformation]:
        """
        Parameters:
            - diagnosis_query: str.
        ---
        from vital.client import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.insurance.search_diagnosis(
            diagnosis_query="diagnosis-query",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/insurance/search/diagnosis"),
            params=remove_none_from_dict({"diagnosis_query": diagnosis_query}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingDiagnosisInformation], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncInsuranceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_payor_info(
        self, *, insurance_name: str, insurance_state: typing.Optional[str] = OMIT
    ) -> typing.List[ClientFacingPayorSearchResponse]:
        """
        Parameters:
            - insurance_name: str.

            - insurance_state: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {"insurance_name": insurance_name}
        if insurance_state is not OMIT:
            _request["insurance_state"] = insurance_state
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/insurance/search/payor"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingPayorSearchResponse], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_diagnosis(self, *, diagnosis_query: str) -> typing.List[ClientFacingDiagnosisInformation]:
        """
        Parameters:
            - diagnosis_query: str.
        ---
        from vital.client import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )
        await client.insurance.search_diagnosis(
            diagnosis_query="diagnosis-query",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v3/insurance/search/diagnosis"),
            params=remove_none_from_dict({"diagnosis_query": diagnosis_query}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ClientFacingDiagnosisInformation], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .activity_v_2_in_db import ActivityV2InDb
from .address import Address
from .appointment_availability_slots import AppointmentAvailabilitySlots
from .appointment_event_status import AppointmentEventStatus
from .appointment_provider import AppointmentProvider
from .appointment_status import AppointmentStatus
from .appointment_type import AppointmentType
from .area_info import AreaInfo
from .attempt_status import AttemptStatus
from .auth_type import AuthType
from .biomarker_result import BiomarkerResult
from .body_v_2_in_db import BodyV2InDb
from .client_activity_response import ClientActivityResponse
from .client_body_response import ClientBodyResponse
from .client_facing_activity import ClientFacingActivity
from .client_facing_api_key import ClientFacingApiKey
from .client_facing_appointment import ClientFacingAppointment
from .client_facing_appointment_cancellation_reason import ClientFacingAppointmentCancellationReason
from .client_facing_appointment_event import ClientFacingAppointmentEvent
from .client_facing_at_home_phlebotomy_order import ClientFacingAtHomePhlebotomyOrder
from .client_facing_at_home_phlebotomy_order_details import ClientFacingAtHomePhlebotomyOrderDetails
from .client_facing_blood_oxygen_timeseries import ClientFacingBloodOxygenTimeseries
from .client_facing_blood_pressure_timeseries import ClientFacingBloodPressureTimeseries
from .client_facing_body import ClientFacingBody
from .client_facing_caffeine_timeseries import ClientFacingCaffeineTimeseries
from .client_facing_calories_active_timeseries import ClientFacingCaloriesActiveTimeseries
from .client_facing_calories_basal_timeseries import ClientFacingCaloriesBasalTimeseries
from .client_facing_cholesterol_timeseries import ClientFacingCholesterolTimeseries
from .client_facing_diagnosis_information import ClientFacingDiagnosisInformation
from .client_facing_distance_timeseries import ClientFacingDistanceTimeseries
from .client_facing_electrocardiogram_voltage_timeseries import ClientFacingElectrocardiogramVoltageTimeseries
from .client_facing_floors_climbed_timeseries import ClientFacingFloorsClimbedTimeseries
from .client_facing_food import ClientFacingFood
from .client_facing_glucose_timeseries import ClientFacingGlucoseTimeseries
from .client_facing_heart_rate import ClientFacingHeartRate
from .client_facing_heart_rate_timeseries import ClientFacingHeartRateTimeseries
from .client_facing_hrv_timeseries import ClientFacingHrvTimeseries
from .client_facing_hypnogram_timeseries import ClientFacingHypnogramTimeseries
from .client_facing_ige_timeseries import ClientFacingIgeTimeseries
from .client_facing_igg_timeseries import ClientFacingIggTimeseries
from .client_facing_lab import ClientFacingLab
from .client_facing_lab_test import ClientFacingLabTest
from .client_facing_loinc import ClientFacingLoinc
from .client_facing_marker import ClientFacingMarker
from .client_facing_marker_complete import ClientFacingMarkerComplete
from .client_facing_meal_response import ClientFacingMealResponse
from .client_facing_mindfulness_minutes_timeseries import ClientFacingMindfulnessMinutesTimeseries
from .client_facing_order import ClientFacingOrder
from .client_facing_order_details import (
    ClientFacingOrderDetails,
    ClientFacingOrderDetails_AtHomePhlebotomy,
    ClientFacingOrderDetails_Testkit,
    ClientFacingOrderDetails_WalkInTest,
)
from .client_facing_order_event import ClientFacingOrderEvent
from .client_facing_patient_details_compatible import ClientFacingPatientDetailsCompatible
from .client_facing_payor_search_response import ClientFacingPayorSearchResponse
from .client_facing_physician import ClientFacingPhysician
from .client_facing_profile import ClientFacingProfile
from .client_facing_provider import ClientFacingProvider
from .client_facing_provider_detailed import ClientFacingProviderDetailed
from .client_facing_provider_with_status import ClientFacingProviderWithStatus
from .client_facing_resource import ClientFacingResource
from .client_facing_respiratory_rate_timeseries import ClientFacingRespiratoryRateTimeseries
from .client_facing_result import ClientFacingResult
from .client_facing_shipment import ClientFacingShipment
from .client_facing_sleep import ClientFacingSleep
from .client_facing_sleep_stream import ClientFacingSleepStream
from .client_facing_source import ClientFacingSource
from .client_facing_sport import ClientFacingSport
from .client_facing_steps_timeseries import ClientFacingStepsTimeseries
from .client_facing_stream import ClientFacingStream
from .client_facing_team import ClientFacingTeam
from .client_facing_test_kit_order_details import ClientFacingTestKitOrderDetails
from .client_facing_testkit_order import ClientFacingTestkitOrder
from .client_facing_user import ClientFacingUser
from .client_facing_user_key import ClientFacingUserKey
from .client_facing_walk_in_order_details import ClientFacingWalkInOrderDetails
from .client_facing_walk_in_test_order import ClientFacingWalkInTestOrder
from .client_facing_water_timeseries import ClientFacingWaterTimeseries
from .client_facing_workout import ClientFacingWorkout
from .client_sleep_response import ClientSleepResponse
from .client_workout_response import ClientWorkoutResponse
from .connected_source_client_facing import ConnectedSourceClientFacing
from .connection_status import ConnectionStatus
from .consent import Consent
from .consent_type import ConsentType
from .day_slots import DaySlots
from .delegated_flow_type import DelegatedFlowType
from .demo_connection_status import DemoConnectionStatus
from .demo_providers import DemoProviders
from .device_v_2_in_db import DeviceV2InDb
from .email_providers import EmailProviders
from .energy import Energy
from .event_destination_preferences import EventDestinationPreferences
from .event_destination_preferences_enabled_item import EventDestinationPreferencesEnabledItem
from .event_destination_preferences_preferred import EventDestinationPreferencesPreferred
from .fallback_time_zone import FallbackTimeZone
from .fats import Fats
from .gender import Gender
from .get_markers_response import GetMarkersResponse
from .get_orders_response import GetOrdersResponse
from .health_insurance_create_request import HealthInsuranceCreateRequest
from .health_insurance_create_request_back_image import (
    HealthInsuranceCreateRequestBackImage,
    HealthInsuranceCreateRequestBackImage_ImageJpeg,
    HealthInsuranceCreateRequestBackImage_ImagePng,
)
from .health_insurance_create_request_front_image import (
    HealthInsuranceCreateRequestFrontImage,
    HealthInsuranceCreateRequestFrontImage_ImageJpeg,
    HealthInsuranceCreateRequestFrontImage_ImagePng,
)
from .health_insurance_create_request_patient_signature_image import (
    HealthInsuranceCreateRequestPatientSignatureImage,
    HealthInsuranceCreateRequestPatientSignatureImage_ImageJpeg,
    HealthInsuranceCreateRequestPatientSignatureImage_ImagePng,
)
from .http_validation_error import HttpValidationError
from .jpeg import Jpeg
from .lab_results_metadata import LabResultsMetadata
from .lab_results_raw import LabResultsRaw
from .lab_results_raw_results import LabResultsRawResults
from .lab_test_collection_method import LabTestCollectionMethod
from .lab_test_sample_type import LabTestSampleType
from .lab_test_status import LabTestStatus
from .last_attempt import LastAttempt
from .libre_config import LibreConfig
from .link_token_exchange_response import LinkTokenExchangeResponse
from .lng_lat import LngLat
from .macros import Macros
from .manual_providers import ManualProviders
from .marker_type import MarkerType
from .meal_in_db_base_client_facing_source import MealInDbBaseClientFacingSource
from .metrics_result import MetricsResult
from .micros import Micros
from .o_auth_providers import OAuthProviders
from .order_status import OrderStatus
from .order_top_level_status import OrderTopLevelStatus
from .paginated_users_response import PaginatedUsersResponse
from .password_providers import PasswordProviders
from .patient_address_compatible import PatientAddressCompatible
from .patient_details import PatientDetails
from .person_details import PersonDetails
from .phlebotomy_area_info import PhlebotomyAreaInfo
from .physician_create_request import PhysicianCreateRequest
from .physician_create_request_base import PhysicianCreateRequestBase
from .physician_create_request_signature_image import (
    PhysicianCreateRequestSignatureImage,
    PhysicianCreateRequestSignatureImage_ImageJpeg,
    PhysicianCreateRequestSignatureImage_ImagePng,
)
from .png import Png
from .post_order_response import PostOrderResponse
from .profile_in_db import ProfileInDb
from .provider_link_response import ProviderLinkResponse
from .providers import Providers
from .raw_activity import RawActivity
from .raw_body import RawBody
from .raw_devices import RawDevices
from .raw_profile import RawProfile
from .raw_sleep import RawSleep
from .raw_workout import RawWorkout
from .region import Region
from .responsible_relationship import ResponsibleRelationship
from .result_type import ResultType
from .shipping_address import ShippingAddress
from .single_resource_statistics import SingleResourceStatistics
from .single_user_resource_response import SingleUserResourceResponse
from .sleep_v_2_in_db import SleepV2InDb
from .source import Source
from .source_auth_type import SourceAuthType
from .source_link import SourceLink
from .source_type import SourceType
from .team_config import TeamConfig
from .time_slot import TimeSlot
from .timeseries_metric_point import TimeseriesMetricPoint
from .timeseries_resource import TimeseriesResource
from .us_address import UsAddress
from .user_refresh_error_response import UserRefreshErrorResponse
from .user_refresh_success_response import UserRefreshSuccessResponse
from .user_resources_response import UserResourcesResponse
from .user_sign_in_token import UserSignInToken
from .user_sign_in_token_response import UserSignInTokenResponse
from .user_sign_in_token_response_sign_in_token import UserSignInTokenResponseSignInToken
from .user_success_response import UserSuccessResponse
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .vital_token_created_response import VitalTokenCreatedResponse
from .workout_v_2_in_db import WorkoutV2InDb

__all__ = [
    "ActivityV2InDb",
    "Address",
    "AppointmentAvailabilitySlots",
    "AppointmentEventStatus",
    "AppointmentProvider",
    "AppointmentStatus",
    "AppointmentType",
    "AreaInfo",
    "AttemptStatus",
    "AuthType",
    "BiomarkerResult",
    "BodyV2InDb",
    "ClientActivityResponse",
    "ClientBodyResponse",
    "ClientFacingActivity",
    "ClientFacingApiKey",
    "ClientFacingAppointment",
    "ClientFacingAppointmentCancellationReason",
    "ClientFacingAppointmentEvent",
    "ClientFacingAtHomePhlebotomyOrder",
    "ClientFacingAtHomePhlebotomyOrderDetails",
    "ClientFacingBloodOxygenTimeseries",
    "ClientFacingBloodPressureTimeseries",
    "ClientFacingBody",
    "ClientFacingCaffeineTimeseries",
    "ClientFacingCaloriesActiveTimeseries",
    "ClientFacingCaloriesBasalTimeseries",
    "ClientFacingCholesterolTimeseries",
    "ClientFacingDiagnosisInformation",
    "ClientFacingDistanceTimeseries",
    "ClientFacingElectrocardiogramVoltageTimeseries",
    "ClientFacingFloorsClimbedTimeseries",
    "ClientFacingFood",
    "ClientFacingGlucoseTimeseries",
    "ClientFacingHeartRate",
    "ClientFacingHeartRateTimeseries",
    "ClientFacingHrvTimeseries",
    "ClientFacingHypnogramTimeseries",
    "ClientFacingIgeTimeseries",
    "ClientFacingIggTimeseries",
    "ClientFacingLab",
    "ClientFacingLabTest",
    "ClientFacingLoinc",
    "ClientFacingMarker",
    "ClientFacingMarkerComplete",
    "ClientFacingMealResponse",
    "ClientFacingMindfulnessMinutesTimeseries",
    "ClientFacingOrder",
    "ClientFacingOrderDetails",
    "ClientFacingOrderDetails_AtHomePhlebotomy",
    "ClientFacingOrderDetails_Testkit",
    "ClientFacingOrderDetails_WalkInTest",
    "ClientFacingOrderEvent",
    "ClientFacingPatientDetailsCompatible",
    "ClientFacingPayorSearchResponse",
    "ClientFacingPhysician",
    "ClientFacingProfile",
    "ClientFacingProvider",
    "ClientFacingProviderDetailed",
    "ClientFacingProviderWithStatus",
    "ClientFacingResource",
    "ClientFacingRespiratoryRateTimeseries",
    "ClientFacingResult",
    "ClientFacingShipment",
    "ClientFacingSleep",
    "ClientFacingSleepStream",
    "ClientFacingSource",
    "ClientFacingSport",
    "ClientFacingStepsTimeseries",
    "ClientFacingStream",
    "ClientFacingTeam",
    "ClientFacingTestKitOrderDetails",
    "ClientFacingTestkitOrder",
    "ClientFacingUser",
    "ClientFacingUserKey",
    "ClientFacingWalkInOrderDetails",
    "ClientFacingWalkInTestOrder",
    "ClientFacingWaterTimeseries",
    "ClientFacingWorkout",
    "ClientSleepResponse",
    "ClientWorkoutResponse",
    "ConnectedSourceClientFacing",
    "ConnectionStatus",
    "Consent",
    "ConsentType",
    "DaySlots",
    "DelegatedFlowType",
    "DemoConnectionStatus",
    "DemoProviders",
    "DeviceV2InDb",
    "EmailProviders",
    "Energy",
    "EventDestinationPreferences",
    "EventDestinationPreferencesEnabledItem",
    "EventDestinationPreferencesPreferred",
    "FallbackTimeZone",
    "Fats",
    "Gender",
    "GetMarkersResponse",
    "GetOrdersResponse",
    "HealthInsuranceCreateRequest",
    "HealthInsuranceCreateRequestBackImage",
    "HealthInsuranceCreateRequestBackImage_ImageJpeg",
    "HealthInsuranceCreateRequestBackImage_ImagePng",
    "HealthInsuranceCreateRequestFrontImage",
    "HealthInsuranceCreateRequestFrontImage_ImageJpeg",
    "HealthInsuranceCreateRequestFrontImage_ImagePng",
    "HealthInsuranceCreateRequestPatientSignatureImage",
    "HealthInsuranceCreateRequestPatientSignatureImage_ImageJpeg",
    "HealthInsuranceCreateRequestPatientSignatureImage_ImagePng",
    "HttpValidationError",
    "Jpeg",
    "LabResultsMetadata",
    "LabResultsRaw",
    "LabResultsRawResults",
    "LabTestCollectionMethod",
    "LabTestSampleType",
    "LabTestStatus",
    "LastAttempt",
    "LibreConfig",
    "LinkTokenExchangeResponse",
    "LngLat",
    "Macros",
    "ManualProviders",
    "MarkerType",
    "MealInDbBaseClientFacingSource",
    "MetricsResult",
    "Micros",
    "OAuthProviders",
    "OrderStatus",
    "OrderTopLevelStatus",
    "PaginatedUsersResponse",
    "PasswordProviders",
    "PatientAddressCompatible",
    "PatientDetails",
    "PersonDetails",
    "PhlebotomyAreaInfo",
    "PhysicianCreateRequest",
    "PhysicianCreateRequestBase",
    "PhysicianCreateRequestSignatureImage",
    "PhysicianCreateRequestSignatureImage_ImageJpeg",
    "PhysicianCreateRequestSignatureImage_ImagePng",
    "Png",
    "PostOrderResponse",
    "ProfileInDb",
    "ProviderLinkResponse",
    "Providers",
    "RawActivity",
    "RawBody",
    "RawDevices",
    "RawProfile",
    "RawSleep",
    "RawWorkout",
    "Region",
    "ResponsibleRelationship",
    "ResultType",
    "ShippingAddress",
    "SingleResourceStatistics",
    "SingleUserResourceResponse",
    "SleepV2InDb",
    "Source",
    "SourceAuthType",
    "SourceLink",
    "SourceType",
    "TeamConfig",
    "TimeSlot",
    "TimeseriesMetricPoint",
    "TimeseriesResource",
    "UsAddress",
    "UserRefreshErrorResponse",
    "UserRefreshSuccessResponse",
    "UserResourcesResponse",
    "UserSignInToken",
    "UserSignInTokenResponse",
    "UserSignInTokenResponseSignInToken",
    "UserSuccessResponse",
    "ValidationError",
    "ValidationErrorLocItem",
    "VitalTokenCreatedResponse",
    "WorkoutV2InDb",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .client_facing_heart_rate import ClientFacingHeartRate
from .client_facing_source import ClientFacingSource

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingActivity(pydantic.BaseModel):
    user_id: str = pydantic.Field(
        description="User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api."
    )
    id: str
    date: dt.datetime = pydantic.Field(
        description="Date of the specified record, formatted as ISO8601 datetime string in UTC 00:00. Deprecated in favour of calendar_date."
    )
    calendar_date: str = pydantic.Field(description="Date of the summary in the YYYY-mm-dd format.")
    calories_total: typing.Optional[float] = pydantic.Field(
        description="Total energy consumption during the day including Basal Metabolic Rate in kilocalories::kilocalories"
    )
    calories_active: typing.Optional[float] = pydantic.Field(
        description="Energy consumption caused by the physical activity of the day in kilocalories::kilocalories"
    )
    steps: typing.Optional[int] = pydantic.Field(description="Total number of steps registered during the day::steps")
    daily_movement: typing.Optional[float] = pydantic.Field(
        description="Deprecated. Daily physical activity as equal meters i.e. amount of walking needed to get the same amount of activity::meters"
    )
    distance: typing.Optional[float] = pydantic.Field(
        description="Distance traveled during activities throughout the day::meters"
    )
    low: typing.Optional[float] = pydantic.Field(
        description="Number of minutes during the day with low intensity activity (e.g. household work)::minutes"
    )
    medium: typing.Optional[float] = pydantic.Field(
        description="Number of minutes during the day with medium intensity activity (e.g. walking)::minutes"
    )
    high: typing.Optional[float] = pydantic.Field(
        description="Number of minutes during the day with high intensity activity (e.g. running)::minutes"
    )
    source: ClientFacingSource = pydantic.Field(description="Source the data has come from.")
    floors_climbed: typing.Optional[int] = pydantic.Field(description="Number of floors climbed by the user::count")
    time_zone: typing.Optional[str] = pydantic.Field(
        description="[DEPRECATED] The time zone full identifier for the data. Example: 'Europe/London'."
    )
    timezone_offset: typing.Optional[int] = pydantic.Field(
        description="Timezone offset from UTC as seconds. For example, EEST (Eastern European Summer Time, +3h) is 10800. PST (Pacific Standard Time, -8h) is -28800::seconds"
    )
    heart_rate: typing.Optional[ClientFacingHeartRate] = pydantic.Field(description="Heart rate daily summary.")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .client_facing_lab import ClientFacingLab
from .client_facing_marker import ClientFacingMarker
from .lab_test_collection_method import LabTestCollectionMethod
from .lab_test_sample_type import LabTestSampleType
from .lab_test_status import LabTestStatus

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingLabTest(pydantic.BaseModel):
    id: str
    slug: str
    name: str
    sample_type: LabTestSampleType
    method: LabTestCollectionMethod
    price: float
    is_active: bool = pydantic.Field(description="Deprecated. Use status instead.")
    status: LabTestStatus
    fasting: typing.Optional[bool] = pydantic.Field(
        description="Defines whether a lab test requires fasting. Only available for Labcorp."
    )
    lab: typing.Optional[ClientFacingLab]
    markers: typing.Optional[typing.List[ClientFacingMarker]]
    is_delegated: typing.Optional[bool] = pydantic.Field(
        description="Denotes whether a lab test requires using non-Vital physician networks. If it does then it's delegated - no otherwise."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .appointment_event_status import AppointmentEventStatus
from .appointment_provider import AppointmentProvider
from .appointment_status import AppointmentStatus
from .appointment_type import AppointmentType
from .client_facing_appointment_event import ClientFacingAppointmentEvent
from .lng_lat import LngLat
from .us_address import UsAddress

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingAppointment(pydantic.BaseModel):
    id: str
    user_id: str
    order_id: str
    address: UsAddress
    location: LngLat
    start_at: dt.datetime = pydantic.Field(description="Time is in UTC")
    end_at: dt.datetime = pydantic.Field(description="Time is in UTC")
    iana_timezone: str
    type: AppointmentType
    provider: AppointmentProvider
    status: AppointmentStatus
    provider_id: str
    can_reschedule: bool
    event_status: AppointmentEventStatus
    event_data: typing.Optional[typing.Dict[str, typing.Any]]
    events: typing.List[ClientFacingAppointmentEvent]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Providers(str, enum.Enum):
    """
    An enumeration.
    """

    OURA = "oura"
    FITBIT = "fitbit"
    GARMIN = "garmin"
    WHOOP = "whoop"
    STRAVA = "strava"
    RENPHO = "renpho"
    PELOTON = "peloton"
    WAHOO = "wahoo"
    ZWIFT = "zwift"
    FREESTYLE_LIBRE = "freestyle_libre"
    FREESTYLE_LIBRE_BLE = "freestyle_libre_ble"
    EIGHT_SLEEP = "eight_sleep"
    WITHINGS = "withings"
    APPLE_HEALTH_KIT = "apple_health_kit"
    MANUAL = "manual"
    IHEALTH = "ihealth"
    GOOGLE_FIT = "google_fit"
    BEURER_API = "beurer_api"
    BEURER_BLE = "beurer_ble"
    OMRON = "omron"
    OMRON_BLE = "omron_ble"
    ONETOUCH_BLE = "onetouch_ble"
    ACCUCHEK_BLE = "accuchek_ble"
    CONTOUR_BLE = "contour_ble"
    DEXCOM = "dexcom"
    DEXCOM_V_3 = "dexcom_v3"
    HAMMERHEAD = "hammerhead"
    MY_FITNESS_PAL = "my_fitness_pal"
    HEALTH_CONNECT = "health_connect"
    POLAR = "polar"
    CRONOMETER = "cronometer"
    KARDIA = "kardia"

    def visit(
        self,
        oura: typing.Callable[[], T_Result],
        fitbit: typing.Callable[[], T_Result],
        garmin: typing.Callable[[], T_Result],
        whoop: typing.Callable[[], T_Result],
        strava: typing.Callable[[], T_Result],
        renpho: typing.Callable[[], T_Result],
        peloton: typing.Callable[[], T_Result],
        wahoo: typing.Callable[[], T_Result],
        zwift: typing.Callable[[], T_Result],
        freestyle_libre: typing.Callable[[], T_Result],
        freestyle_libre_ble: typing.Callable[[], T_Result],
        eight_sleep: typing.Callable[[], T_Result],
        withings: typing.Callable[[], T_Result],
        apple_health_kit: typing.Callable[[], T_Result],
        manual: typing.Callable[[], T_Result],
        ihealth: typing.Callable[[], T_Result],
        google_fit: typing.Callable[[], T_Result],
        beurer_api: typing.Callable[[], T_Result],
        beurer_ble: typing.Callable[[], T_Result],
        omron: typing.Callable[[], T_Result],
        omron_ble: typing.Callable[[], T_Result],
        onetouch_ble: typing.Callable[[], T_Result],
        accuchek_ble: typing.Callable[[], T_Result],
        contour_ble: typing.Callable[[], T_Result],
        dexcom: typing.Callable[[], T_Result],
        dexcom_v_3: typing.Callable[[], T_Result],
        hammerhead: typing.Callable[[], T_Result],
        my_fitness_pal: typing.Callable[[], T_Result],
        health_connect: typing.Callable[[], T_Result],
        polar: typing.Callable[[], T_Result],
        cronometer: typing.Callable[[], T_Result],
        kardia: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Providers.OURA:
            return oura()
        if self is Providers.FITBIT:
            return fitbit()
        if self is Providers.GARMIN:
            return garmin()
        if self is Providers.WHOOP:
            return whoop()
        if self is Providers.STRAVA:
            return strava()
        if self is Providers.RENPHO:
            return renpho()
        if self is Providers.PELOTON:
            return peloton()
        if self is Providers.WAHOO:
            return wahoo()
        if self is Providers.ZWIFT:
            return zwift()
        if self is Providers.FREESTYLE_LIBRE:
            return freestyle_libre()
        if self is Providers.FREESTYLE_LIBRE_BLE:
            return freestyle_libre_ble()
        if self is Providers.EIGHT_SLEEP:
            return eight_sleep()
        if self is Providers.WITHINGS:
            return withings()
        if self is Providers.APPLE_HEALTH_KIT:
            return apple_health_kit()
        if self is Providers.MANUAL:
            return manual()
        if self is Providers.IHEALTH:
            return ihealth()
        if self is Providers.GOOGLE_FIT:
            return google_fit()
        if self is Providers.BEURER_API:
            return beurer_api()
        if self is Providers.BEURER_BLE:
            return beurer_ble()
        if self is Providers.OMRON:
            return omron()
        if self is Providers.OMRON_BLE:
            return omron_ble()
        if self is Providers.ONETOUCH_BLE:
            return onetouch_ble()
        if self is Providers.ACCUCHEK_BLE:
            return accuchek_ble()
        if self is Providers.CONTOUR_BLE:
            return contour_ble()
        if self is Providers.DEXCOM:
            return dexcom()
        if self is Providers.DEXCOM_V_3:
            return dexcom_v_3()
        if self is Providers.HAMMERHEAD:
            return hammerhead()
        if self is Providers.MY_FITNESS_PAL:
            return my_fitness_pal()
        if self is Providers.HEALTH_CONNECT:
            return health_connect()
        if self is Providers.POLAR:
            return polar()
        if self is Providers.CRONOMETER:
            return cronometer()
        if self is Providers.KARDIA:
            return kardia()



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .client_facing_at_home_phlebotomy_order_details import ClientFacingAtHomePhlebotomyOrderDetails
from .client_facing_test_kit_order_details import ClientFacingTestKitOrderDetails
from .client_facing_walk_in_order_details import ClientFacingWalkInOrderDetails


class ClientFacingOrderDetails_WalkInTest(ClientFacingWalkInOrderDetails):
    type: typing_extensions.Literal["walk_in_test"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ClientFacingOrderDetails_Testkit(ClientFacingTestKitOrderDetails):
    type: typing_extensions.Literal["testkit"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ClientFacingOrderDetails_AtHomePhlebotomy(ClientFacingAtHomePhlebotomyOrderDetails):
    type: typing_extensions.Literal["at_home_phlebotomy"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


ClientFacingOrderDetails = typing.Union[
    ClientFacingOrderDetails_WalkInTest, ClientFacingOrderDetails_Testkit, ClientFacingOrderDetails_AtHomePhlebotomy
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AuthType(str, enum.Enum):
    """
    An enumeration.
    """

    PASSWORD = "password"
    OAUTH = "oauth"
    EMAIL = "email"

    def visit(
        self,
        password: typing.Callable[[], T_Result],
        oauth: typing.Callable[[], T_Result],
        email: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AuthType.PASSWORD:
            return password()
        if self is AuthType.OAUTH:
            return oauth()
        if self is AuthType.EMAIL:
            return email()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .client_facing_api_key import ClientFacingApiKey
from .delegated_flow_type import DelegatedFlowType
from .team_config import TeamConfig

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingTeam(pydantic.BaseModel):
    id: str
    name: str
    svix_app_id: typing.Optional[str]
    client_id: typing.Optional[str]
    client_secret: typing.Optional[str]
    airtable_api_key: typing.Optional[str]
    airtable_base_id: typing.Optional[str]
    webhook_secret: typing.Optional[str]
    api_key: typing.Optional[str]
    api_keys: typing.Optional[typing.List[ClientFacingApiKey]]
    configuration: typing.Optional[TeamConfig]
    testkits_texts_enabled: bool
    lab_tests_patient_communication_enabled: bool
    lab_tests_patient_sms_communication_enabled: bool
    lab_tests_patient_email_communication_enabled: bool
    logo_url: typing.Optional[str]
    delegated_flow: DelegatedFlowType

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingResource(str, enum.Enum):
    """
    An enumeration.
    """

    PROFILE = "profile"
    ACTIVITY = "activity"
    SLEEP = "sleep"
    BODY = "body"
    WORKOUTS = "workouts"
    WORKOUT_STREAM = "workout_stream"
    CONNECTION = "connection"
    ORDER = "order"
    RESULT = "result"
    APPOINTMENT = "appointment"
    GLUCOSE = "glucose"
    HEARTRATE = "heartrate"
    HRV = "hrv"
    HYPNOGRAM = "hypnogram"
    IGE = "ige"
    IGG = "igg"
    BLOOD_OXYGEN = "blood_oxygen"
    BLOOD_PRESSURE = "blood_pressure"
    CHOLESTEROL = "cholesterol"
    DEVICE = "device"
    WEIGHT = "weight"
    FAT = "fat"
    MEAL = "meal"
    WATER = "water"
    CAFFEINE = "caffeine"
    MINDFULNESS_MINUTES = "mindfulness_minutes"
    STEPS = "steps"
    CALORIES_ACTIVE = "calories_active"
    DISTANCE = "distance"
    FLOORS_CLIMBED = "floors_climbed"
    RESPIRATORY_RATE = "respiratory_rate"
    VO_2_MAX = "vo2_max"
    CALORIES_BASAL = "calories_basal"
    STRESS_LEVEL = "stress_level"
    ELECTROCARDIOGRAM_VOLTAGE = "electrocardiogram_voltage"
    SLEEP_STREAM = "sleep_stream"

    def visit(
        self,
        profile: typing.Callable[[], T_Result],
        activity: typing.Callable[[], T_Result],
        sleep: typing.Callable[[], T_Result],
        body: typing.Callable[[], T_Result],
        workouts: typing.Callable[[], T_Result],
        workout_stream: typing.Callable[[], T_Result],
        connection: typing.Callable[[], T_Result],
        order: typing.Callable[[], T_Result],
        result: typing.Callable[[], T_Result],
        appointment: typing.Callable[[], T_Result],
        glucose: typing.Callable[[], T_Result],
        heartrate: typing.Callable[[], T_Result],
        hrv: typing.Callable[[], T_Result],
        hypnogram: typing.Callable[[], T_Result],
        ige: typing.Callable[[], T_Result],
        igg: typing.Callable[[], T_Result],
        blood_oxygen: typing.Callable[[], T_Result],
        blood_pressure: typing.Callable[[], T_Result],
        cholesterol: typing.Callable[[], T_Result],
        device: typing.Callable[[], T_Result],
        weight: typing.Callable[[], T_Result],
        fat: typing.Callable[[], T_Result],
        meal: typing.Callable[[], T_Result],
        water: typing.Callable[[], T_Result],
        caffeine: typing.Callable[[], T_Result],
        mindfulness_minutes: typing.Callable[[], T_Result],
        steps: typing.Callable[[], T_Result],
        calories_active: typing.Callable[[], T_Result],
        distance: typing.Callable[[], T_Result],
        floors_climbed: typing.Callable[[], T_Result],
        respiratory_rate: typing.Callable[[], T_Result],
        vo_2_max: typing.Callable[[], T_Result],
        calories_basal: typing.Callable[[], T_Result],
        stress_level: typing.Callable[[], T_Result],
        electrocardiogram_voltage: typing.Callable[[], T_Result],
        sleep_stream: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingResource.PROFILE:
            return profile()
        if self is ClientFacingResource.ACTIVITY:
            return activity()
        if self is ClientFacingResource.SLEEP:
            return sleep()
        if self is ClientFacingResource.BODY:
            return body()
        if self is ClientFacingResource.WORKOUTS:
            return workouts()
        if self is ClientFacingResource.WORKOUT_STREAM:
            return workout_stream()
        if self is ClientFacingResource.CONNECTION:
            return connection()
        if self is ClientFacingResource.ORDER:
            return order()
        if self is ClientFacingResource.RESULT:
            return result()
        if self is ClientFacingResource.APPOINTMENT:
            return appointment()
        if self is ClientFacingResource.GLUCOSE:
            return glucose()
        if self is ClientFacingResource.HEARTRATE:
            return heartrate()
        if self is ClientFacingResource.HRV:
            return hrv()
        if self is ClientFacingResource.HYPNOGRAM:
            return hypnogram()
        if self is ClientFacingResource.IGE:
            return ige()
        if self is ClientFacingResource.IGG:
            return igg()
        if self is ClientFacingResource.BLOOD_OXYGEN:
            return blood_oxygen()
        if self is ClientFacingResource.BLOOD_PRESSURE:
            return blood_pressure()
        if self is ClientFacingResource.CHOLESTEROL:
            return cholesterol()
        if self is ClientFacingResource.DEVICE:
            return device()
        if self is ClientFacingResource.WEIGHT:
            return weight()
        if self is ClientFacingResource.FAT:
            return fat()
        if self is ClientFacingResource.MEAL:
            return meal()
        if self is ClientFacingResource.WATER:
            return water()
        if self is ClientFacingResource.CAFFEINE:
            return caffeine()
        if self is ClientFacingResource.MINDFULNESS_MINUTES:
            return mindfulness_minutes()
        if self is ClientFacingResource.STEPS:
            return steps()
        if self is ClientFacingResource.CALORIES_ACTIVE:
            return calories_active()
        if self is ClientFacingResource.DISTANCE:
            return distance()
        if self is ClientFacingResource.FLOORS_CLIMBED:
            return floors_climbed()
        if self is ClientFacingResource.RESPIRATORY_RATE:
            return respiratory_rate()
        if self is ClientFacingResource.VO_2_MAX:
            return vo_2_max()
        if self is ClientFacingResource.CALORIES_BASAL:
            return calories_basal()
        if self is ClientFacingResource.STRESS_LEVEL:
            return stress_level()
        if self is ClientFacingResource.ELECTROCARDIOGRAM_VOLTAGE:
            return electrocardiogram_voltage()
        if self is ClientFacingResource.SLEEP_STREAM:
            return sleep_stream()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.client_facing_profile import ClientFacingProfile
from ...types.http_validation_error import HttpValidationError
from ...types.raw_profile import RawProfile

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ProfileClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, user_id: str, *, provider: typing.Optional[str] = None) -> ClientFacingProfile:
        """
        Get Daily profile for user_id

        Parameters:
            - user_id: str.

            - provider: typing.Optional[str]. Provider oura/strava etc
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/summary/profile/{user_id}"),
            params=remove_none_from_dict({"provider": provider}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingProfile, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_raw(self, user_id: str, *, provider: typing.Optional[str] = None) -> RawProfile:
        """
        Get Daily profile for user_id

        Parameters:
            - user_id: str.

            - provider: typing.Optional[str]. Provider oura/strava etc
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/summary/profile/{user_id}/raw"),
            params=remove_none_from_dict({"provider": provider}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RawProfile, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncProfileClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, user_id: str, *, provider: typing.Optional[str] = None) -> ClientFacingProfile:
        """
        Get Daily profile for user_id

        Parameters:
            - user_id: str.

            - provider: typing.Optional[str]. Provider oura/strava etc
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/summary/profile/{user_id}"),
            params=remove_none_from_dict({"provider": provider}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ClientFacingProfile, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_raw(self, user_id: str, *, provider: typing.Optional[str] = None) -> RawProfile:
        """
        Get Daily profile for user_id

        Parameters:
            - user_id: str.

            - provider: typing.Optional[str]. Provider oura/strava etc
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v2/summary/profile/{user_id}/raw"),
            params=remove_none_from_dict({"provider": provider}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RawProfile, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions

from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingCaloriesBasalTimeseries(pydantic.BaseModel):
    id: typing.Optional[int] = pydantic.Field(description="Deprecated")
    timezone_offset: typing.Optional[int] = pydantic.Field(
        description="Time zone UTC offset in seconds. Positive offset indicates east of UTC; negative offset indicates west of UTC; and null indicates the time zone information is unavailable at source."
    )
    type: typing.Optional[str] = pydantic.Field(
        description="The reading type of the measurement. This is applicable only to Cholesterol, IGG and IGE."
    )
    unit: typing_extensions.Literal["kcal"]
    timestamp: dt.datetime = pydantic.Field(description="The timestamp of the measurement.")
    value: float = pydantic.Field(description="Basal Metabolic Rate at the time or interval::kilocalories")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .client_facing_source import ClientFacingSource
from .client_facing_sport import ClientFacingSport

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClientFacingWorkout(pydantic.BaseModel):
    user_id: str = pydantic.Field(
        description="User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api."
    )
    id: str
    title: typing.Optional[str] = pydantic.Field(description="Title given for the workout")
    timezone_offset: typing.Optional[int] = pydantic.Field(
        description="Timezone offset from UTC as seconds. For example, EEST (Eastern European Summer Time, +3h) is 10800. PST (Pacific Standard Time, -8h) is -28800::seconds"
    )
    average_hr: typing.Optional[int] = pydantic.Field(description="Average heart rate during workout::bpm")
    max_hr: typing.Optional[int] = pydantic.Field(description="Max heart rate during workout::bpm")
    distance: typing.Optional[float] = pydantic.Field(description="Distance travelled during workout::meters")
    calendar_date: str = pydantic.Field(
        description="Date of the workout summary in the YYYY-mm-dd format. This generally matches the workout start date."
    )
    time_start: dt.datetime = pydantic.Field(description="Start time of the workout::time")
    time_end: dt.datetime = pydantic.Field(description="End time of the workout::time")
    calories: typing.Optional[float] = pydantic.Field(description="Calories burned during the workout::kCal")
    sport: typing.Optional[ClientFacingSport] = pydantic.Field(description="Sport's name")
    hr_zones: typing.Optional[typing.List[int]] = pydantic.Field(
        description="Time in seconds spent in different heart rate zones <50%, 50-60%, 60-70%, 70-80%, 80-90%, 90%+. Due to rounding errors, it's possible that summing all values is different than the total time of the workout. Not available for all providers::seconds"
    )
    moving_time: typing.Optional[int] = pydantic.Field(description="Time spent active during the workout::seconds")
    total_elevation_gain: typing.Optional[float] = pydantic.Field(
        description="Elevation gain during the workout::meters"
    )
    elev_high: typing.Optional[float] = pydantic.Field(description="Highest point of elevation::meters")
    elev_low: typing.Optional[float] = pydantic.Field(description="Lowest point of elevation::meters")
    average_speed: typing.Optional[float] = pydantic.Field(
        description="Average speed during workout in m/s::meters/sec"
    )
    max_speed: typing.Optional[float] = pydantic.Field(description="Max speed during workout in m/s::meters/sec")
    average_watts: typing.Optional[float] = pydantic.Field(description="Average watts burned during exercise::watts")
    device_watts: typing.Optional[float] = pydantic.Field(description="Watts burned during exercise::watts")
    max_watts: typing.Optional[float] = pydantic.Field(description="Max watts burned during exercise::watts")
    weighted_average_watts: typing.Optional[float] = pydantic.Field(
        description="Weighted average watts burned during exercise::watts"
    )
    map: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(
        description="Map of workouts encoded as polyline"
    )
    provider_id: str = pydantic.Field(description="Provider ID given for that specific workout")
    source: ClientFacingSource = pydantic.Field(description="Source the data has come from.")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
